#ifndef HELPER_H_INCLUDED
#define HELPER_H_INCLUDED


// Cripto++ Header
#include "aes.h"
#include "modes.h"
#include "sha.h"
#include "osrng.h"
#include "files.h"
#include "filters.h"
#include "md5.h"
#include "hex.h"
#include "hmac.h"
#include "base64.h"
#include "dh.h"
#include "secblock.h"
#include <vector>

#include "dhjson.h"

using namespace CryptoPP;
using namespace std;

/**
    DIFFIE HELLMAN constant for keys generations
*/
#define DH_MODULUS_DEC "1717183979661295860112291519931784809019042025337056958695697\
60169920539808075437788747086722975900425740754301098468647941395164593810074170462799\
60806249302198928583741681554872103587437854812123605094852822941613958557156899806658\
6304075565145536350296006867635076744949977849997684222020336013226588207303"


/** DIFFIE HELLMAN Base */
#define DH_BASE  "2"

/** lenght of aes key */
#define LEN 128

/**
    Utility for conversion from a byte* to char*
    \param byte* buffer, byte for converting in char*
    \param int len, len of the buffer
    \return the char* conversion
*/
char* bytes(byte* buffer, int len = 16);

/**
    Tokenize a string
*/
void tokenize(const string& str, vector<string>& tokens, const string& delimiters = " ");

/**
    Utility for conversion from a char * to SecByteBlock
    \param const char *hex, the hex strign
    \return the SecByteBlock conversion
*/
SecByteBlock DecodeHexString(const char *hex);

/**
    Returns a random number used for Diffie Hellman key exchange
    \return Returns a random number
*/
CryptoPP::RandomPool & GlobalRNG();


/**
    This function generate the key agreement.

    Funzione che dato la priv key di un utente e la pubblica di
    un altro utente, genera la key aggregata corrispondente

    \param char* DHprivateKey, owner private key
    \param char* DHpublicKey, user public key
    \param char* OWNprivateKey, key derived from owner PIN

    \return key agreement key
*/
byte* KeyAgreement(char* DHprivateKey, char* DHpublicKey, char* OWNprivateKey);


// la chiave DHprivateKey è in chiaro
/**
    This function generate the key agreement.

    Funzione che dato la priv key di un utente e la pubblica di
    un altro utente, genera la key aggregata corrispondente

    \param char* DHprivateKey, owner private key
    \param char* DHpublicKey, user public key
    \param char* OWNprivateKey, key derived from owner PIN

    \return key agreement key in plaintext
*/
byte* PlainKeyAgreement(char* DHprivateKey, char* DHpublicKey, char* OWNprivateKey);


/**
    Utility for conversion from const char* to byte*
*/
byte* hexChartoHexBytes(const char* str );


/**
    Generate a random label to calculate tokens
    \param int len, lenght of the result label (default 32)
    \param int start, the lesser value of the string
    \param int range, value range for the label

    \return generated label
*/
char* generateRandomLabel(int len = 0x20, int start = 'a', int range = 'z'-'a');


/**
    Generate a random key

    \param int len, lenght of the result key (default 32)
    \param int start, the lesser value of the string
    \param int range, value range for the key

    \return generated key
*/
char* generateRandomKey(int len = 0x20, int start = 0, int range = 16);


/**
    Calcolate key for decryption operations
    \param char* jsonin, path of json file
    \param char* pin, owner pin
*/
byte* prepareDecryption(char* jsonin,char* pin);
byte* prepareDecryptionHBEL(char* jsonin, char* pin);


/**
    Calcolate key for encryption operations
    \param char* jsonin, path of json file
    \param char* pin, owner pin
*/
byte* prepareEncryption(char* filein, char* fileout, char* pin);

/**
    This function use the Atallah idea for creating token.
    \param char* keySource
    \param char* keyDestination
    \param char* label
    \return byte*
*/
byte* createToken(byte* keySource, char* keyDestination, char* label);
byte* createTokenHBEL(byte* keySource, char* keyDestination, char* label);


/**
    Iterate throw the tokens and build the key
    \param char* keySource
    \param char* keyDestination
    \param char* label
    \return the key for encryption/decryption
*/
byte* rebuildKey(byte* keySource, byte* token, char* label);


/**
    Decode string using AES 128 bit
    \param string source, the string source
    \param byte* key, the AES 128 bit
    \return string in plaintext
*/
string decodeAESKey(string source, byte* key);

/**
    Encode string using AES 128 bit
    \param SecByteBlock source, the string source
    \param byte* key, the AES 128 bit
    \return string in chiper format
*/
string encodeAESKey(SecByteBlock source, byte* key);

/**
    Encode string using AES 128 bit
    \param string source, the string source
    \param byte* key, the AES 128 bit
    \return string in chiper format
*/
string encodeAESKey(string source, byte* key);


/**
    Generate Diffie-Hellman Keys
    \param string pin, source string to generate DH keys
    \return string in JSON format that contains public key, private key and the has
    of the pin

*/
string generateDiffieHellmanKeysPair(string pin);

/**
    Generate Owner Key starting from the PIN
    \param PIN, the string for key generation
*/
byte* generateOwnerKey(string pin);

/**
    This function encrypt file
    \param const char* password, encryption password
    \param const char* inputFileName, input file name,
    \param const char* outputFileName, output filename
*/
void encryptFile(byte* password,const char* inputFileName,
                 const char* outputFileName);

/**
    This function decrypt file
    \param const char* password, encryption password
    \param const char* inputFileName, input file name,
    \param const char* outputFileName, output filename
*/

void decryptFile(byte* password, const char*inputFileName,
                 const char* outputFileName);

int decryptSEL(char* password, char*inputFileName,
                 char* outputFileName);

/**
    This function calculate the hash of the string
    \param string str,

    \return the hash string
*/
byte* hash(string str);
byte* hash(byte* key);
#endif // HELPER_H_INCLUDED

